/*  $Id: component_options.h,v 1.42 2003/07/20 22:36:15 grumbel Exp $
**
**  ClanLib Game SDK
**  Copyright (C) 2003  The ClanLib Team
**  For a total list of contributers see the file CREDITS.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Lesser General Public License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
**
*/

//! clanGUI="Framework"
//! header=gui.h

#ifndef header_styleoptions
#define header_styleoptions

#include "../Core/System/clanstring.h"
#include "../Core/System/error.h"
#include <map>
#include <string>
#include <stdio.h>

#ifdef WIN32
#define snprintf _snprintf
#endif

//: Sets the options for components
class CL_ComponentOptions
{
//! Attributes:
public:
	typedef std::multimap<std::string, std::string> options_t;
	typedef options_t::iterator iterator;
	typedef options_t::const_iterator const_iterator;

	//: Exists
	bool exists(const std::string &option_name) const
	{
		return options.find(CL_String::to_lower(option_name)) != options.end();
	}

	//: Count
	int count(const std::string &option_name) const
	{
		return (int) options.count(CL_String::to_lower(option_name));
	}

	//: Getting the value of a unexisting option is undefined, so use the 'exists' function
	//: first to check if the option exists
	const std::string &get_value(const std::string &option_name, int offset=0) const
	{ 
		std::string lc_name = CL_String::to_lower(option_name);
		std::multimap<std::string, std::string>::const_iterator it = options.find(lc_name);
		if (it == options.end())
		{
			char buf[100];
			snprintf(buf, 99, "Missing component option '%s'", option_name.c_str());
			buf[99] = 0;
			cl_throw_error(buf);
		}
		for (;offset>0;offset--)
		{
			it++;
			if (it == options.end())
			{
				char buf[100];
				snprintf(buf, 99, "Missing component option '%s'", option_name.c_str());
				buf[99] = 0;
				cl_throw_error(buf);
			}
		}
		return (*it).second; 
	}

	const std::string &operator[](const std::string &option_name) const
	{
		return get_value(option_name, 0);
	}

	//: Get value as int
	int get_value_as_int(const std::string &option_name, int offset=0) const
	{
		const std::string &val = get_value(option_name, offset);
		return atoi(val.c_str()); 
	}

	//: Get value as bool
	bool get_value_as_bool(const std::string &option_name, int offset=0) const
	{
		const std::string s = CL_String::to_lower(get_value(option_name, offset));

		if(s == "yes" ||
			s == "true" ||
			s == "on" ||
			s == "1")
			return true;
		else if(s == "no" ||
			s == "false" ||
			s == "off" ||
			s == "0")
			return false;

		char buf[200];
		snprintf(
			buf, 199,
			"Invalid bool value '%s' in component option '%s'", 
			get_value(option_name, offset).c_str(),
			option_name.c_str());
		buf[199] = 0;
		cl_throw_error(buf);
		return false;
	}
	
//! Operations:
public:
	//: Add option
	void add_option(const std::string &option_name, const std::string &option_value)
	{
		options.insert(options_t::value_type(CL_String::to_lower(option_name), option_value));
	}
	
	//: Add option
	void add_option(const std::string &option_name, int option_value)
	{
		add_option(option_name, CL_String::from_int(option_value));
	}
	
	//: Remove option
	void remove_option(const std::string &option_name)
	{
		std::string lc_name = CL_String::to_lower(option_name);
		std::multimap<std::string, std::string>::iterator it = options.find(lc_name);
		while (it != options.end())
		{
			if (it->first != option_name) break;
			std::multimap<std::string, std::string>::iterator it2 = it;
			++it;
			options.erase(it2);
		}
	}

	//: Clear
	void clear()
	{
		options.clear();
	}

	iterator begin() { return options.begin(); }
	iterator end() { return options.end(); }

	const_iterator begin() const { return options.begin(); }
	const_iterator end() const { return options.end(); }

	//: Options
	options_t options;
};

#endif
